                          ;NES Play ASM file



 
;memory map 
;
;0000-1fffh - system RAM
;2000-4fffh - not used (cannot be used due to reg. addresses)
;5000-5f7fh - control ROM
;5f80-5fefh - control RAM
;5ff0h      - timer, low byte
;5ff1h      - timer, high byte
;5ff2h      - memory lock
;5ff3h      - memory lock
;5ff4h      - LCD write address (RS=0)
;5ff5h      - LCD write address (RS=1)
;5ff6h      - reset IRQ flag
;5ff7h      - inputs
;5ff8-5fffh - bankswitch control
;6000-7fffh - SRAM emulation
;8000-fffbh - PRG ROM emulation
;fffc-ffffh - reset and IRQ vectors 
 
 .org 0

card_buf:    .block 130
card_temp:   .block 1
card_temp2:  .block 1
card_xor:    .block 1
x_temp:      .block 1
card_add:    .block 2
card_img:    .block 5 ;for card 5A,52,5C,5D,00
temp_add:    .block 2

d_reg:       
d_reg0:      .block 1
d_reg1:      .block 1

sum_0:       .block 1
sum_1:       .block 1

 .org 05f80h


 .org 05000h    


lcd_write:   .equ 05ff4h
timer_lo:    .equ 05ff0h
timer_hi:    .equ 05ff1h
reset_irq:   .equ 05ff6h
mlock_set:   .equ 05ff3h
mlock_rst:   .equ 05ff2h
inputs:      .equ 05ff7h

card_in:     .equ 05ff6h
bank_adds:   .equ 05ff8h
card_out:    .equ 04016h


start:       sei
             cld
             ldx #0ffh
             txs

             lda #0c0h
             sta 04017h
             
             ldx #13h
             lda #0

cl_lp:       sta 04000h,x
             dex
             bne cl_lp
             sta 04015h


             ldy #0
             sty 0
             lda #7
             sta 1
             tya

f_lp:        sta (0),y
             dey
             bne f_lp
             dec 1
             bpl f_lp        ;clear RAM
             
             
             lda reset_irq   ;reset IRQ latch
             ldx #0

             
fgf:         dey             
             nop
             nop
             bne fgf
             dex
             bne fgf
             
             
             
             lda #01fh
             jsr lcd_ins
             lda #00fh
             jsr lcd_ins
             lda #001h
             jsr lcd_ins    ;init LCD

             
             lda dataloc
             sta card_add
             lda dataloc+1
             sta card_add+1  ;start of card data
             
             lda #0
             sta d_reg0
             lda #053h
             sta d_reg1      ;start of data to be stored
             
sd_lpj:      ldy #0

sd_lpj2:     lda (d_reg),y
             sta card_buf,y
             clc
             adc sum_0
             sta sum_0
             lda #00h
             adc sum_1
             sta sum_1
             iny
             cpy #080h
             bne sd_lpj2
             jsr card_wrt
             bcc no_c_err
             jmp c_err

no_c_err:    lda #080h
             clc
             adc d_reg0
             sta d_reg0
             lda #000h
             adc d_reg1
             sta d_reg1
             clc
             inc card_add
             bne no_i
             inc card_add+1

no_i:        lda #02
             jsr lcd_ins
             lda card_add
             and #3
             tax
             lda sho_tick,x
             jsr lcd_dat
             lda d_reg1
             cmp #05bh
             bne sd_lpj





             
             
             
             lda 05300h
             jsr sho_hex
             lda 057ffh
             jsr sho_hex

             lda #' '
             jsr lcd_dat
             lda #'L'
             jsr lcd_dat
             lda #'D'
             jsr lcd_dat
             lda #' '
             jsr lcd_dat


             lda sum_1
             jsr sho_hex
             lda sum_0
             jsr sho_hex


dead:        jmp dead

c_err:       lda #'E'
             jsr lcd_dat
             jsr lcd_dat
             jmp dead


sho_tick:    .db 02dh,08ch,07ch,02fh

lcd_dat:     sta lcd_write+1
             

l1:          bit lcd_write
             bmi l1
             rts

lcd_ins:     sta lcd_write
             jmp l1


sho_hex:     pha
             ror a
             ror a
             ror a
             ror a
             jsr s_h1
             pla

s_h1:        and #00fh
             tax
             lda hex_tab,x
             jmp lcd_dat



hex_tab:     .db "0123456789ABCDEF"



card_wrt:    lda #000h
             sta card_out    ;CLK, SEL, Dout all high
             
             jsr ms_16       ;wait 16ms
             
             lda #004h       ;lower select to enable card
             sta card_out
             
             lda #081h
             jsr send_byte
             lda #057h
             jsr send_byte
             lda #000h
             jsr send_byte
             cmp #05ah
             bne cw_err
             lda #000h
             jsr send_byte
             cmp #05dh
             bne cw_err
             lda card_add+1
             jsr send_byte
             cmp #000h
             bne cw_err
             lda card_add
             jsr send_byte
             cmp card_add+1
             bne cw_err
             eor card_add
             sta card_xor
             ldx #0
             
g_loop:      stx x_temp
             lda card_buf,x
             jsr send_byte
             bcc *+3
             rts
             ldx x_temp
             lda card_buf,x
             eor card_xor
             sta card_xor
             inx
             cpx #128
             bne g_loop
             
             lda card_xor
             jsr send_byte
             lda #000h
             jsr send_byte
             cmp #05ch
             bne cw_err
             lda #000h
             jsr send_byte
             cmp #05dh
             bne cw_err
             lda #000h
             jsr send_bytel
             cmp #'G'
             bne cw_err
             lda #000h
             sta card_out
             clc
             rts

cw_err:      lda #000h
             sta card_out
             sec
             rts

card_err:    lda #0ffh
             rts


send_byte:   sta card_temp
             ldx #4
             
sb_loop:     lda #007h
             clc
             ror card_temp
             bcc no_set
             lda #006h
             
no_set:      sta card_out     ;set/clear output bit & pull clk low
             nop
             bit card_in
             clc
             bvc no_ll
             sec

no_ll:       ror card_temp2
             and #005h
             sta card_out     ;set clock to lock in value
             dex
             bne sb_loop

             lda inputs    ;reset latch
             ldx #4

sb_loop2:    lda #007h
             clc
             ror card_temp
             bcc no_set2
             lda #006h
             
no_set2:     sta card_out     ;set/clear output bit & pull clk low
             bit card_in
             clc
             bvc no_ll2
             sec

no_ll2:      ror card_temp2
             and #005h
             sta card_out     ;set clock to lock in value
             dex
             bne sb_loop2
             ldx #0
             ldy #0
             
card_dl:     bit card_in
             bpl no_lwt
             dex
             bne card_dl
             dey
             bne card_dl
             sec              ;not OK flag
             lda card_temp2
             rts

no_lwt:      lda card_temp2
             clc              ;OK flag
             rts


send_bytel:  sta card_temp
             ldx #8
             
sb_loopl:    lda #007h
             clc
             ror card_temp
             bcc no_setl
             lda #006h
             
no_setl:     sta card_out     ;set/clear output bit & pull clk low
             nop
             bit card_in
             clc
             bvc no_lll
             sec

no_lll:      ror card_temp2
             and #005h
             sta card_out     ;set clock to lock in value
             dex
             bne sb_loopl
             lda card_temp2
             clc              ;OK flag
             rts              ;no ACK check
             

ms_16:       ldx #19
             
ff:          ldy #0h      ;3  \

gh:          dey          ;2  |1542 cycles
             nop          ;2  |
             bne gh       ;2  |
             dex          ;2  |
             bne ff       ;2  /
             rts
             




 .fill 052feh-*,0ffh

dataloc:     


 .end

